package edu.travel.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import com.nimbusds.jose.JWSObject;
import edu.travel.FinalParam;
import edu.travel.util.EncryptUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.ParseException;
import java.util.*;
/**
 * LoginFilter 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/3/11
 */

@Component
public class LoginFilter implements Filter {

//    @Autowired
//    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
            throws IOException, ServletException {

        HttpServletRequest request = (HttpServletRequest) servletRequest;
        HttpServletResponse response = (HttpServletResponse) servletResponse;

        // 获取请求路径
        String requestURI = request.getRequestURI();

        // **放行登录相关接口**
        if (requestURI.startsWith("/baby/login")) {
            filterChain.doFilter(request, response);
            return;
        }

        // 获取 token
        String token = request.getHeader("Authorization");

        if (StringUtils.isBlank(token) || token.equals("null")) {
            filterChain.doFilter(request, response);
            return;
        }

        // 替换 Bearer 头
        String realToken = token.replace("Bearer ", "");
        try {
            JWSObject jwsObject = JWSObject.parse(realToken);
            String userStr = jwsObject.getPayload().toString();
            JSONObject jsonObject = JSON.parseObject(userStr);

            // 提取权限信息
            Object object = jsonObject.get(FinalParam.AUTHORITY_CLAIM_NAME);
            List<String> authorities = new ArrayList<>();

            if (object != null) {
                if (object instanceof JSONArray) {
                    authorities = ((JSONArray) object).toJavaList(String.class);
                } else {
                    String authoritiesStr = object.toString();
                    JSONArray authoritiesArray = JSON.parseArray(authoritiesStr);
                    authorities = authoritiesArray.toJavaList(String.class);
                }
            }

            // 提取用户名和客户端ID
            String principal = jsonObject.getObject("user_name", String.class);
            String clientId = jsonObject.getObject("client_id", String.class);

            // 封装到 Map
            Map<String, Object> map = new HashMap<>();
            map.put("authorities", authorities);
            map.put("client_id", clientId);
            map.put("principal", principal);

            // 生成新的 token（Base64 编码）
            String newToken = EncryptUtil.encodeUTF8StringBase64(JSON.toJSONString(map));

            // 将 token 添加到请求头
            HttpServletRequestWrapper requestWrapper = new HttpServletRequestWrapper(request) {
                @Override
                public String getHeader(String name) {
                    if ("token".equals(name)) {
                        return newToken;
                    }
                    return super.getHeader(name);
                }
            };

            filterChain.doFilter(requestWrapper, response);

        } catch (ParseException e) {
            e.printStackTrace();
            filterChain.doFilter(request, response);
        }
    }
}
